==> shelf_route-0.1.2%2B1.tar.gz: lib/shelf_route.dart <==
// Copyright (c) 2014, The Shelf Route project authors.
// Please see the AUTHORS file for details.
// All rights reserved. Use of this source code is governed by
// a BSD 2-Clause License that can be found in the LICENSE file.

library shelf_route;

import 'package:shelf/shelf.dart';
import 'package:path/path.dart' as p;
import 'package:matcher/matcher.dart';
import 'src/preconditions.dart';
import 'src/constraint.dart';

/// A [Handler] that routes to other handlers based on the incoming request
/// (path and method). Note there can be several layers of routers to facilitate
/// modularity.
///
/// When the Router routes a request to a handler it adjusts the requests as follows:
/// * the routes path is removed from the start of the requests pathInfo
/// * the routes path is added to the end of the requests scriptName
///
/// e.g if original request had path info of `/banking' and scriptName was
/// `/abc` then when routing a request of `/banking/accounts` the new request
/// past to the handler will have a pathInfo of `/accounts` and scriptName of
/// `/abc/banking`
///
///
Router router() => new Router();

PathVariable pathVariable(String name, {Type type}) =>
    new PathVariable(name, type: type);

const String _PATH_VARIABLE_PARAM_PREFIX = 'shelf.route.';
//Request addPathVariables(Request request, )

Map<String, Object> getPathVariables(Request request) {
  // TODO(anders): replace with request.extraParams when supported in shelf
  final params = request.headers;
  final keys = params.keys.where(
      (k) => k.startsWith(_PATH_VARIABLE_PARAM_PREFIX));
  
  final pathVars = <String, Object>{};
  keys.forEach((k) {
    pathVars[k.substring(_PATH_VARIABLE_PARAM_PREFIX.length)] = params[k];
  });
  
  return pathVars;
}

class Router {
  final List<_Route> _routes = <_Route>[];
  final _Route _fallbackRoute;

  Router._internal(this._fallbackRoute);

  Router([Handler fallbackHandler])
      : this._fallbackRoute = new _Route(fallbackHandler != null ?
          fallbackHandler : _send404, null, null, []);

  void addRoute(Handler handler, {String path, String method,
    List<PathVariable> pathVariables: const [] }) {
    _routes.add(new _Route(handler, path, method, pathVariables));
  }

  void get(String path, Handler handler,
      { List<PathVariable> pathVariables: const [] }) {

    addRoute(handler, path: path, method: 'GET', pathVariables: pathVariables);
  }

  void post(String path, Handler handler,
      { List<PathVariable> pathVariables: const [] }) {

    addRoute(handler, path: path, method: 'POST', pathVariables: pathVariables);
  }

  Router addChildRouter(String path) {
    final child = new Router._internal(_fallbackRoute);
    addRoute(child.handler, path: path);
    return child;
  }

  Handler get handler => _handleRequest;

  dynamic _handleRequest(Request request) {
    final route = _routes.firstWhere((r) => r.canHandle(request),
        orElse: () => _fallbackRoute);
    return route.handle(request) ;
  }
}


class _Route {
  final Handler handler;
  final List<_RoutePathSegment> _pathSegments;
  final String method;
  final Map<String, PathVariable> _pathVariableMap;

  _Route._internal(this.handler, this._pathSegments, this.method,
      this._pathVariableMap) {
    checkThat(handler, isNotNull, 'handler cannot be null');
    checkThat(_pathSegments, isNotNull, '_pathSegments cannot be null');
    checkThat(_pathVariableMap, isNotNull, '_pathVariableMap cannot be null');
  }

  factory _Route(Handler handler, String path, String method,
      List<PathVariable> pathVariables) {
    checkThat(pathVariables, isNotNull, 'pathVariables cannot be null');

    final pathVariableMap = {};

    pathVariables.forEach((pv) {
      pathVariableMap[pv.name] =  pv;
    });

    _RoutePathSegment create(String seg) {
      if (PathVariable.isPathVariable(seg)) {
        final name = seg.substring(1);
        var pathVariable = pathVariableMap[name];
        if (pathVariable == null) {
          pathVariable = new PathVariable(name);
          pathVariableMap[name] = pathVariable;
        }
        return new _VariablePathSegment(pathVariable);
      }
      else {
        return new _NormalPathSegment(seg);
      }
    }

    final Iterable<_RoutePathSegment> segs =
        path == null ? [] : p.split(path).map(create).toList();

    final pathSegments =
        segs.length == 1 && segs.first.matches('/') ? const [] : segs;

    return new _Route._internal(handler, pathSegments, method,
        pathVariableMap);

  }

  bool canHandle(Request request) {
    final pathInfo = request.pathInfo;

    return
        (method == null || request.method == method)
        &&
        (_pathSegments.isEmpty || _isWithin(pathInfo));
  }

  bool _isWithin(String pathInfo) {
    final List<String> requestSegs = p.split(pathInfo);
    if (_pathSegments.length > requestSegs.length) {
      return false;
    }

    for(int i = 0; i < _pathSegments.length; i++) {
      if (!_pathSegments[i].matches(requestSegs[i])) {
        return false;
      }
    }
    return true;
  }

  dynamic handle(Request request) {
    // create a new request with the pathInfo and scriptName adjusted relative
    // to route path
    final newRequest = _createNewRequest(request);

    return handler(newRequest);
  }

  Request _createNewRequest(Request request) {
    final requestSegs = p.split(request.pathInfo);
    final newRequestSegs = requestSegs.skip(_pathSegments.length);
    var newPathInfo = _adjustPath(p.joinAll(newRequestSegs));
    final skipCount = request.scriptName.isNotEmpty && requestSegs.isNotEmpty &&
        requestSegs.first == '/' ? 1 : 0;
    final extraScriptPath = requestSegs.skip(skipCount).take(_pathSegments.length);
    var newScriptName = p.joinAll(p.split(request.scriptName)
        ..addAll(extraScriptPath));

    if (newScriptName == '/') {
      newScriptName = '';
    }

    if (newPathInfo == '' && newScriptName == '') {
      newPathInfo = '/';
    }

    var newHeaders = request.headers;
    if (containsPathVariables) {
      newHeaders = {}..addAll(request.headers);

      for (int i = 0; i < _pathSegments.length; i++) {
        final seg = _pathSegments[i];
        if (seg is _VariablePathSegment) {
          final pathVariable = seg.variable;
          // TODO: header values are strings but hopefully request params will be objects
  //        final value = pathVariable.convert(requestSegs.elementAt(i));
          newHeaders['$_PATH_VARIABLE_PARAM_PREFIX${pathVariable.name}'] =
              requestSegs.elementAt(i);
        }
      }
    }

//    final newRequest = request.copyWith(pathInfo: newPathInfo,
//        scriptName: newScriptName);

    return new Request(newPathInfo, request.queryString,
        request.method,
        newScriptName, request.protocolVersion, request.requestedUri,
        newHeaders, body: request.read());

  }

  bool get containsPathVariables => _pathVariableMap.isNotEmpty;

  String _adjustPath(String path) {
    if (path.isNotEmpty && !path.startsWith('/')) {
      path = p.join('/', path);
    }

    return path;
  }

}

Response _send404(Request req) {
  return new Response.notFound("Not Found");
}


class PathVariable {
  final String name;
  final Type type;
  final List<Constraint> constraints;

  PathVariable(this.name, {this.type, this.constraints: const []}) {
    checkThat(name, isNotNull);
  }

  static bool isPathVariable(String seg) => seg.startsWith(':');

  String toString() => ':$name';
}

abstract class _RoutePathSegment {
  bool matches(String pathSegment);
}

class _NormalPathSegment implements _RoutePathSegment {
  final String value;

  _NormalPathSegment(this.value) {
    checkThat(value, isNotNull, 'value cannot be null');
  }

  @override
  bool matches(String pathSegment) => value == pathSegment;

  String toString() => value;
}

class _VariablePathSegment implements _RoutePathSegment {
  final PathVariable variable;
  _VariablePathSegment(this.variable) {
    checkThat(variable, isNotNull, 'variable cannot be null');
  }

  @override
  bool matches(String pathSegment) {
    return true; // TODO: only for string types with no constraints
  }

  String toString() => variable.toString();
}

==> shelf_route-0.2.0%2B1.tar.gz: lib/shelf_route.dart <==
// Copyright (c) 2014, The Shelf Route project authors.
// Please see the AUTHORS file for details.
// All rights reserved. Use of this source code is governed by
// a BSD 2-Clause License that can be found in the LICENSE file.

library shelf_route;

import 'package:shelf/shelf.dart';
import 'package:path/path.dart' as p;
import 'package:matcher/matcher.dart';
import 'src/preconditions.dart';
import 'src/constraint.dart';

/// A [Handler] that routes to other handlers based on the incoming request
/// (path and method). Note there can be several layers of routers to facilitate
/// modularity.
///
/// When the Router routes a request to a handler it adjusts the requests as follows:
/// * the routes path is removed from the start of the requests pathInfo
/// * the routes path is added to the end of the requests scriptName
///
/// e.g if original request had path info of `/banking' and scriptName was
/// `/abc` then when routing a request of `/banking/accounts` the new request
/// past to the handler will have a pathInfo of `/accounts` and scriptName of
/// `/abc/banking`
///
///
Router router() => new Router();

PathVariable pathVariable(String name, {Type type}) =>
    new PathVariable(name, type: type);

const String _PATH_VARIABLE_PARAM_PREFIX = 'shelf.route.';
//Request addPathVariables(Request request, )

Map<String, Object> getPathVariables(Request request) {
  // TODO(anders): replace with request.extraParams when supported in shelf
  final params = request.headers;
  final keys = params.keys.where(
      (k) => k.startsWith(_PATH_VARIABLE_PARAM_PREFIX));

  final pathVars = <String, Object>{};
  keys.forEach((k) {
    pathVars[k.substring(_PATH_VARIABLE_PARAM_PREFIX.length)] = params[k];
  });

  return pathVars;
}

class Router {
  final List<_Route> _routes = <_Route>[];
  final _Route _fallbackRoute;

  Router._internal(this._fallbackRoute);

  Router([Handler fallbackHandler])
      : this._fallbackRoute = new _Route(fallbackHandler != null ?
          fallbackHandler : _send404, null, null, []);

  void addRoute(Handler handler, {String path, String method,
    List<PathVariable> pathVariables: const [] }) {
    _routes.add(new _Route(handler, path, method, pathVariables));
  }

  void get(String path, Handler handler,
      { List<PathVariable> pathVariables: const [] }) {

    addRoute(handler, path: path, method: 'GET', pathVariables: pathVariables);
  }

  void post(String path, Handler handler,
      { List<PathVariable> pathVariables: const [] }) {

    addRoute(handler, path: path, method: 'POST', pathVariables: pathVariables);
  }

  Router addChildRouter(String path) {
    final child = new Router._internal(_fallbackRoute);
    addRoute(child.handler, path: path);
    return child;
  }

  Handler get handler => _handleRequest;

  dynamic _handleRequest(Request request) {
    final route = _routes.firstWhere((r) => r.canHandle(request),
        orElse: () => _fallbackRoute);
    return route.handle(request) ;
  }
}


class _Route {
  final Handler handler;
  final List<_RoutePathSegment> _pathSegments;
  final String method;
  final Map<String, PathVariable> _pathVariableMap;

  _Route._internal(this.handler, this._pathSegments, this.method,
      this._pathVariableMap) {
    checkThat(handler, isNotNull, 'handler cannot be null');
    checkThat(_pathSegments, isNotNull, '_pathSegments cannot be null');
    checkThat(_pathVariableMap, isNotNull, '_pathVariableMap cannot be null');
  }

  factory _Route(Handler handler, String path, String method,
      List<PathVariable> pathVariables) {
    checkThat(pathVariables, isNotNull, 'pathVariables cannot be null');

    final pathVariableMap = {};

    pathVariables.forEach((pv) {
      pathVariableMap[pv.name] =  pv;
    });

    _RoutePathSegment create(String seg) {
      if (PathVariable.isPathVariable(seg)) {
        final name = seg.substring(1);
        var pathVariable = pathVariableMap[name];
        if (pathVariable == null) {
          pathVariable = new PathVariable(name);
          pathVariableMap[name] = pathVariable;
        }
        return new _VariablePathSegment(pathVariable);
      }
      else {
        return new _NormalPathSegment(seg);
      }
    }

    final Iterable<_RoutePathSegment> segs =
        path == null ? [] : Uri.parse(path).pathSegments.map(create).toList();
//        path == null ? [] : p.split(path).map(create).toList();

    final pathSegments =
        segs.length == 1 && segs.first.matches('/') ? const [] : segs;

    return new _Route._internal(handler, pathSegments, method,
        pathVariableMap);

  }

  bool canHandle(Request request) {
    return
        (method == null || request.method == method)
        &&
        (_pathSegments.isEmpty || _isWithin(request.url));
  }

  bool _isWithin(Uri url) {
    final List<String> requestSegs = url.pathSegments;
    if (_pathSegments.length > requestSegs.length) {
      return false;
    }

    for(int i = 0; i < _pathSegments.length; i++) {
      if (!_pathSegments[i].matches(requestSegs[i])) {
        return false;
      }
    }
    return true;
  }

  dynamic handle(Request request) {
    // create a new request with the pathInfo and scriptName adjusted relative
    // to route path
    final newRequest = _createNewRequest(request);

    return handler(newRequest);
  }

  Request _createNewRequest(Request request) {

    final requestSegs = request.url.pathSegments;
    final newRequestSegs = requestSegs.skip(_pathSegments.length);
    var newPathInfo = _adjustPath(p.joinAll(newRequestSegs));
    final skipCount = request.scriptName.isNotEmpty && requestSegs.isNotEmpty &&
        requestSegs.first == '/' ? 1 : 0;
    final extraScriptPath = requestSegs.skip(skipCount).take(_pathSegments.length);
    var newScriptName = _adjustPath(p.joinAll(p.split(request.scriptName)
        ..addAll(extraScriptPath)));

    if (newPathInfo == '' && newScriptName == '') {
      newPathInfo = '/';
    }

    var newHeaders = request.headers;
    if (containsPathVariables) {
      newHeaders = {}..addAll(request.headers);

      for (int i = 0; i < _pathSegments.length; i++) {
        final seg = _pathSegments[i];
        if (seg is _VariablePathSegment) {
          final pathVariable = seg.variable;
          // TODO: header values are strings but hopefully request params will be objects
  //        final value = pathVariable.convert(requestSegs.elementAt(i));
          newHeaders['$_PATH_VARIABLE_PARAM_PREFIX${pathVariable.name}'] =
              requestSegs.elementAt(i);
        }
      }
    }

//    final newRequest = request.copyWith(pathInfo: newPathInfo,
//        scriptName: newScriptName);
    return new Request(request.method, request.requestedUri,
        protocolVersion: request.protocolVersion, headers: newHeaders,
        url: Uri.parse(newPathInfo), scriptName: newScriptName,
        body: request.read());

  }

  bool get containsPathVariables => _pathVariableMap.isNotEmpty;

  String _adjustPath(String path) {
    if (path.isNotEmpty && !path.startsWith('/')) {
      path = p.join('/', path);
    }

    return path;
  }

}

Response _send404(Request req) {
  return new Response.notFound("Not Found");
}


class PathVariable {
  final String name;
  final Type type;
  final List<Constraint> constraints;

  PathVariable(this.name, {this.type, this.constraints: const []}) {
    checkThat(name, isNotNull);
  }

  static bool isPathVariable(String seg) => seg.startsWith(':');

  String toString() => ':$name';
}

abstract class _RoutePathSegment {
  bool matches(String pathSegment);
}

class _NormalPathSegment implements _RoutePathSegment {
  final String value;

  _NormalPathSegment(this.value) {
    checkThat(value, isNotNull, 'value cannot be null');
  }

  @override
  bool matches(String pathSegment) => value == pathSegment;

  String toString() => value;
}

class _VariablePathSegment implements _RoutePathSegment {
  final PathVariable variable;
  _VariablePathSegment(this.variable) {
    checkThat(variable, isNotNull, 'variable cannot be null');
  }

  @override
  bool matches(String pathSegment) {
    return pathSegment != null && pathSegment != '/'; // TODO: only for string types with no constraints
  }

  String toString() => variable.toString();
}
